<?php
/**
 * login.php - Login controller
 *
 * @author $Author: dtong $
 * @version $Id: login.php,v 1.48 2012/02/09 11:00:48 dtong Exp $
 * @copyright Copyright (c) 2009, Tiller Software Co., Ltd.
*/

class Login extends CI_Controller {
    const SECONDS_IN_MINUTE = 60;
    //DB has 500 bytes so make 2 less to ensure it never gets overfilled
    const MAX_DB_IP_STORAGE = 498;
    //This is the max length of a IPV6 address.
    const MAX_IP_ADDRESS_LENGTH = 45;
    const IPLIST_DELIMETER = ',';
    //5 failed login within 10 minutes will lock the account for 10 minutes
    private $failed_lockout_count = 5;
    //In minutes
    private $lockout_period = 15;
    private $is_localuser = TRUE;
    private $add_external_user = FALSE;

    function __construct() {
		parent::__construct();
		osa_header_nocache();
      $this->load->helper(array('form', 'url'));
      $this->load->library('form_validation');
      $this->load->library('cms/cms_captcha');
      osa_load_lang('login');
      $this->failed_lockout_count = $this->config->item('cms_failed_lockout_count');
      $this->lockout_period = $this->config->item('cms_lockout_period');
    }

   function index() {
       if ( $this->login_model->isAuthenticated() ) {
           if ( $this->session->flashdata('LOGINPAGE_NO_REDIRECT_HOME') != '1') {
                 redirect('home');
              exit;
           }
       }
      $this->form_validation->set_rules('username', 'Username',
         'trim|required|min_length[1]|strtolower');
      //Can't use a variable to set lenght of password, has to be hardcoded here
      $this->form_validation->set_rules('password', 'Password',
         'required|min_length[1]');
      $this->form_validation->set_rules('captcha_code', 'Confirmation code', 'trim');
      $msg = FALSE;
      if ($this->form_validation->run() == FALSE) {
          //Just to make sure we are clean
          $username = set_value('username');
          $password = set_value('password');
          if ( $username != '' ) {
              $msg = lang('login_missingpassword');
          }
          $sess_username = $this->session->flashdata('AUTH_FAILED_USERNAME');
          if ( $sess_username ) {
              $username = $sess_username;
              $msg = lang('login_invalidlogin');
          }
          if ( $this->session->flashdata(CMS_SESSION_FAILED_LOGIN_LOCKOUT) == TRUE ) {
              $msg = sprintf(lang('login_lockout'), $this->failed_lockout_count, $this->lockout_period);
          }
          $msg_mode= FALSE;
          if ( $this->config->item('cms_manage_mode') ) {
               $msg_mode = lang('login_managemode') ;
          }
          if (  $this->config->item('cms_sitestop') ) {
              $msg_mode = lang('gen_maintenance_msg');
          }

          $sess_managemode = $this->session->flashdata('AUTH_FAILED_MANAGE_MODE');
          if ( $sess_managemode ) {
               $msg = '';  //$msg_mode = lang('login_managemode');
          }

          osa_del_treecookies();
          $data = array();
          //Pass array of array for view within view
          $data['data'] = array('username'=>osa_des_decrypt($username), 'msg'=>$msg, 'msg_mode'=>$msg_mode);
          // Check captcha usage
          if ( $this->config->item('cms_login_use_captcha') &&
                  $this->session->flashdata(CMS_SESSION_USE_CAPTCHA) ) {
              $data['captcha_image'] = $this->cms_captcha->create_captcha();
              $data['captcha_error'] = $this->session->flashdata(CMS_SESSION_CAPTCHA_ERROR);
              $data['use_captcha'] = TRUE;
              if ( !empty($data['captcha_error']) && $msg == lang('login_invalidlogin') ) {
                  //No need to print invalid user name or passwd if we have captcha error msg
                  $data['data']['msg'] = '';
                }
          }
          // End of check captcha usage
          //main_login_view contains multiple views
          $this->load->view('login_m_view', $data);
      }
      else {
      	$username = set_value('username');
      	$password = set_value('password');
      	$username = osa_des_decrypt($username);
      	$password = osa_des_decrypt($password);
      	$captcha_code = set_value('captcha_code');
      	if ( empty($captcha_code) ) {
      		$captcha_code = FALSE;
      	}
      	if ( $this->lockout_check($username) && $this->check_capthcha($username, $captcha_code) &&
      		  ($login_obj = $this->login_model->authenticate($username, $password)) ) {
      		 //Don't update for magic password
      		 if ( !property_exists($login_obj, 'cms_stopmode_passthrough') || !$login_obj->cms_stopmode_passthrough ) {
      		 	$this->success_update($username);
      		 }
      		 $this->load->library('cms/Cms_scheduler');
      		 redirect('home', 'location', 302, TRUE);
             $this->cms_scheduler->login_event();
             exit;
          }
          else {
              // check guest login
              $user = $this->login_model->getDbUser($username, TRUE);
              if ( $user === FALSE ){
                  //This can redirect and exit upon successful remote guest login.
                  //Right now, only remote LDAP is supported. This code will need to be changed
                  //if we need to support remote guest login other thasn LDAP
                  $this->_guest_login($username, $password, $captcha_code, 'ldap');
              }
                $this->session->set_flashdata('AUTH_FAILED_USERNAME', $username);
                $this->failed_update($username);
                redirect('');
            exit;
        }
      }
   }

   function logout() {
      $this->login_model->destroyLogin();
      @osa_php_session_destroy();
      osa_del_treecookies();
      redirect('');
      exit;
   }

   function guest() {
       if ( $this->config->item('cms_guest_open_access') ) {
           $this->login_model->setGuestSession();
           redirect('home');
         exit;
       }
        redirect('');
      exit;
   }

   function key() {
       $key = osa_des_session_key();
       if ( !empty($key) ) {
           //Session has the key client has already so just say success
           echo 'success';
           exit;
       }
       //This generates a new key since the session already expired.
       $this->load->helper('des_helper');
       echo stringToHex(osa_des_random_key());
       exit;
   }

   //Check to see if the account is locked out
   private function lockout_check($username, $set_session=TRUE) {
       if ( !$this->config->item('cms_failed_login_lockout_check') || $this->config->item('cms_login_use_captcha') ) {
           return TRUE;
       }
       //This is the same call in login_model->authenticate() and the result is already cached
       //if we are here
        $user = $this->_get_user($username);
        if ( !is_object($user) ) {
            osa_log(__METHOD__, 'Failed to get user object.', $username);
            return FALSE;
        }

        if ( $this->match_ip($user->successips, $_SERVER['REMOTE_ADDR'])) {
            //The IP did login successfully in the past so skip the lockout check
            return TRUE;
        }

        $dif = time() - strtotime($user->lastfailedlogin);
        if ( $user->lastfailedlogin && $user->failedlogincount >= $this->failed_lockout_count &&
              $dif < ($this->lockout_period * self::SECONDS_IN_MINUTE) ) {
            if ( $set_session ) {
                //Informs the account lockout
                $this->session->set_flashdata(CMS_SESSION_FAILED_LOGIN_LOCKOUT, TRUE);
            }
            return FALSE;
        }
        return TRUE;
   }

   //Updates the user record for failed attempt
    private function failed_update($username) {
        $user = $this->_get_user($username);
        if ( !$user ) {
            return FALSE;
        }
        $dif = time() - strtotime($user->lastfailedlogin);
        if ( $user->failedlogincount >  $this->failed_lockout_count &&
              $dif < ($this->lockout_period * self::SECONDS_IN_MINUTE) ) {
            //It is already locked so no need to update the DB
            return TRUE;
        }
        $failedips = $user->failedips;
        $failedlogincount = $user->failedlogincount + 1;
        if ( $failedlogincount <= 0 ) {
            //This means recent failed login attempt after a successful attempt so reset it.
            //A successful login will make the failed login count into a negative number.
            $failedlogincount = 1;
        }
        $lastfailedlogin = osa_dbdate();
        if ( $dif >= ($this->lockout_period * self::SECONDS_IN_MINUTE) ) {
            //A failed attempt after the lockout period expires.
            $failedlogincount = 1;
        }
        //It is not really used, just for tracking purpose
        $failedips = $this->add_ip2list($failedips, $_SERVER['REMOTE_ADDR']);
        $record = new user_update_class($username, $failedlogincount, $failedips, FALSE, FALSE, $lastfailedlogin);
        return $this->_update_user($user, $record);
   }

   //Updates the user record for success login
   private function success_update($username) {
       $user = $this->_get_user($username);
        if ( !$user ) {
            osa_log(__METHOD__, 'Failed to get user object.', $username);
            return FALSE;
        }
        $failedlogincount = $user->failedlogincount;
        if ( $failedlogincount > 0 ) {
            //This for keeping track the last fail count after the first successful login
            //A negative number means the last failelogincount
            $failedlogincount *= -1;
        }
        elseif ( $failedlogincount < 0 ) {
            //This will clear the negative number on the second successful login
            $failedlogincount = 0;
        }
        $lastlogin = osa_dbdate();
        $failedips = NULL;
        $successips = $this->add_ip2list($user->successips, $_SERVER['REMOTE_ADDR']);
        $record = new user_update_class($username, $failedlogincount, $failedips, $successips, $lastlogin);
        return $this->_update_user($user, $record);
   }

   //Returns true if $ip is in $iplist, false otherwise. This works for IPv4 and IPv6.
   //This is a full IP matching and if user is using dynamic IP then this is useless.
   //This is done to prevent DoS attach but only if user coming from the same static IP(s).
   //TODO: Should allow subnet matching for IPv4 and IPv6 since it is not likely brute force and DoS attacks
   //          would come from the same subnet. Thinking to use class B for IPv4, for IPv6???
   private function match_ip($iplist, $ip) {
       return osa_in_itemlist($ip, $iplist);
   }

   //Returns a new list with $ip added to $iplist. This will purge old ips from $iplist if not enough storage.
   //This is not specific to any field in the user table.
   //This can mix IPv4 and IPv6 addresses
   private function add_ip2list($iplist, $ip) {
       return osa_add_item2list($iplist, $ip, self::MAX_IP_ADDRESS_LENGTH, self::MAX_DB_IP_STORAGE, self::IPLIST_DELIMETER);
   }

   // verify CAPTCHA (confirmation code) from input
   private function check_capthcha($username, $captcha_code) {
       // login_lockout and login_use_captcha both TRUE can be happen, so use operand AND (&&)
       if ( !$this->config->item('cms_login_use_captcha') && $this->config->item('cms_failed_login_lockout_check') ) {
            return TRUE;
       }
       $user = $this->_get_user($username);
       $use_captcha = FALSE;
       if ( !is_object($user) ) {
            return FALSE;
        }
        $dif = time() - strtotime($user->lastfailedlogin);
        // greater than or eqaul (>=) logic use same as in lockout_check function
        if ( property_exists($user, 'lastfailedlogin') && $user->failedlogincount >= $this->failed_lockout_count &&
              $dif < ($this->lockout_period * self::SECONDS_IN_MINUTE) ) {
            $this->session->set_flashdata(CMS_SESSION_USE_CAPTCHA , TRUE);
            $use_captcha = TRUE;
       }
        if ( empty($captcha_code) && $use_captcha ) {
            $this->session->set_flashdata(CMS_SESSION_CAPTCHA_ERROR , lang('login_captcha_empty'));
            return FALSE;
        }
        $is_valid = $this->cms_captcha->check_valid_key($captcha_code);
        if ( $is_valid ){
            // disable captcha
            $this->session->set_flashdata(CMS_SESSION_USE_CAPTCHA , FALSE);
            return TRUE;
        }
        if ( !$is_valid && $use_captcha ) {
            $this->session->set_flashdata(CMS_SESSION_CAPTCHA_ERROR , lang('login_captcha_error'));
            return FALSE;
        }
        return TRUE;
   }

   private function _guest_login($username, $password, $captcha_code, $authtype){
        $authobj = auth_factory::factory($authtype);
        // ldap auth
       if ( $this->config->item('cms_guest_ldap_access') ) {
           // disable local auth
           $this->is_localuser = FALSE;
            if ( $this->check_capthcha($username, $captcha_code) && $authobj->authenticate($username, $password, FALSE) ) {
                if ( !$this->login_model->setGuestSession() ) {
                    return FALSE;
                }
                $this->success_update($username);
                redirect('home');
             exit;
            }
      } // END OF ldap auth
      return FALSE;
   }

   private function _get_user($username) {
       if ( !$username ){
           return FALSE;
       }
       // normal auth
       if ( $this->is_localuser ){
           $user = $this->login_model->getDbUser($username, TRUE);
       }
       // external user auth
       else {
           $user = $this->login_model->get_user_external($username);
            if ( !$user ) {
                $this->add_external_user = TRUE;
                $userobj = new user_update_class($username);
                return $userobj;
            }
       }
       if ( is_object($user) ) {
            return $user;
        }
       return FALSE;
   }

   private function _update_user($userobj, $newrecord){
       if ( !$userobj || !$newrecord ){
           return FALSE;
       }
       if ( $this->is_localuser ){
            return $this->login_model->update_user($userobj->username, $newrecord);
       }
       else { // external user
           // add
           if ( $this->add_external_user ) {
                return $this->login_model->add_user_external($newrecord);
            }
            // update
           return $this->login_model->update_user_external($userobj->username, $newrecord);
       }
       return FALSE;
   }
}

//Class to support various object update and insert. It is designed to undset class properties that are not used
//so the data in the database won't get affected.
class user_update_class {
    //Used at all 3 places
    public $username;
    //Used at at least 2 places
    public $failedlogincount;
    public $failedips;
    public $successips;
    //Used at only only 1 place
    public $lastlogin;
    public $lastfailedlogin;

    function __construct($username, $failedlogincount=0, $failedips=FALSE,
                                $successips=FALSE, $lastlogin=FALSE, $lastfailedlogin=FALSE) {
        $this->username = $username;
        $this->failedips = $failedips;
        $this->failedlogincount = $failedlogincount;
        $this->successips = $successips;
        $this->lastlogin = $lastlogin;
        $this->lastfailedlogin = $lastfailedlogin;
        //Unset anyone that is not used (FALSE) and unset
        foreach ($this as $key=>$value) {
            if ( $value === FALSE ) {
                unset ($this->$key);
            }
        }
    }
}
?>